%{

// Directives to the compiler and C declarations
#include <stdio.h>
#include "./lib/syntaxTree.h"
#define YYERROR_VERBOSE 1

// The lexer
extern int yylex();
extern int yyerror(char *);

// The syntax tree: an external global variable.
extern stNode *tree;

%}

// Tokens, given by the lexer
%token IDENTIFIER
%token NUMBER
%token INT
%token FLOAT
%token bEGIN
%token END
%token VARDEC

/* Each rule is a pointer to the structure stNode. Working in this manner, each time a rule is matched, we create a new
   node of the tree and we put it in the right position */
%type <nodep> Body
%type <nodep> StatementList
%type <nodep> VarDec
%type <nodep> VarList
%type <nodep> NameList
%type <nodep> Name
%type <nodep> Statement
%type <nodep> Assignment
%type <nodep> Expr
%type <nodep> Term
%type <nodep> Factor

// Possible types of yylval
%union {
  struct _stNode *nodep;
}

%%

Program
  : bEGIN ProgramName Body END ProgramName
    { tree = createStNode("Program"); 
      appendChild(tree,$3);
    }
  ;

Body
  : VarDec ';' StatementList
    { /* Body is called $$, and its type is pointer to a struct stNode.
         When the rule is matched, we create a new instance of this struct, using the function createStNode, provided by
         the library */
      $$ = createStNode("Body");
      /* Then we append, as child of node Body ($$), the nodes associated to VarDec and StatementList, respectively $1 and $3 */
      appendChild($$,$3);
      appendChild($$,$1);
    }
  ;

VarDec
  : VARDEC VarList
    { $$ = createStNode("VarDec");
      appendChild($$,$2);
    }
  ;

VarList
  : Type NameList
    { /* The case of list handling must be carefully analyzed. 
         To do so, we could refer to an abstract syntax of a list, for example:
                            1. LIST -> e
                            2. LIST -> e,LIST
         The list (that contains elements 'e') will be generated by recursion, applying the rule
         number 2. Then, the derivation is closed, using the rule number one.
         Now, let we consider the syntax tree associated to a derivation:
         using LR parsing technique, first of all the parser recognizes the subtree associated
         to rule number 1 (to be convinced, try to derive a syntax tree of a little list
         by hand, and then try to apply the LR parsing technique, explyciting the stack).

         It implies that the we must create the first node of the list IN THE SEMANTIC
         ACTION OF THIS RULE.
         So, we create the VarList ($$) using our library function "createNode". Then we
         append to the list its first child, NameList ($2).
       */
      $$ = createStNode("VarList"); 
      appendChild($$,$2);
    }
  | Type NameList VarList
    { /* We append to the list ($3) an element ($2). We can notice that the list $3 is the one
         previously created by using the first rule!
         Then we propagate the value assigning $3 to $$ */
      appendChild($3,$2); 
      $$ = $3;
    }
  ;

NameList
  : Name
    { $$ = createStNode("NameList");
      appendChild($$,$1);
    }
  | Name ',' NameList
    { appendChild($3,$1); 
      $$ = $3;
    }
  ;

StatementList
  : Statement ';'
    { $$ = createStNode("StatementList");
      appendChild($$,$1);
    }
  | Statement ';' StatementList
    { appendChild($3,$1); 
      $$ = $3;
    }
  ;

Statement
  : Assignment
    { $$ = createStNode("Statement");
      appendChild($$,$1);
    }
  ;

Assignment
  : Name '=' Expr
    { $$ = createStNode("Assignment");
      appendChild($$,$1);
      appendChild($$,$3);
    }
  ;

Expr
  : Term '+' Expr
    { $$ = createStNode("Expression"); 
      appendChild($$,$1);
      appendChild($$,$3);
    }
  | Term '-' Expr
    { $$ = createStNode("Expression");
      appendChild($$,$1);
      appendChild($$,$3);
    }
  | Term
    { $$ = createStNode("Expression");
      appendChild($$,$1);
    }
  ;

Term
  : Term '*' Factor
    { $$ = createStNode("Term");
      appendChild($$,$1);
      appendChild($$,$3);
    }
  | Factor
    { $$ = createStNode("Term");
      appendChild($$,$1);
    }
  ;

Factor
  : '(' Expr ')'
    { $$ = createStNode("Factor");
      appendChild($$,$2);
    }
  | NUMBER
    { $$ = createStNode("Factor"); }
  | Name
    { $$ = createStNode("Factor"); 
      appendChild($$,$1);
    }
  ;

ProgramName
  : IDENTIFIER
  ;

Name
  : IDENTIFIER
    { $$ = createStNode("Name");}
  ;

Type
  : INT 
  | FLOAT
  ;

  

